@@ -66,6 +66,9 @@ class Pair:
         self.delta = 1.
         if ('delta' in kwargs): self.delta = kwargs['delta']
                        
+        # if pol specified set
+        if ('pol' in kwargs): self.pol = kwargs['pol']
+                       
         # if no args make synthetic
         if len(args) == 0: 
             self.x, self.y = _synth(**kwargs)               
@@ -232,7 +235,7 @@ class Pair:
     def data(self):
         return np.vstack((self.x,self.y))
 
-    def pol(self):
+    def get_pol(self):
         """Return principal component orientation"""
         # rotate to zero
         rot = self.cmpvecs.T
@@ -276,6 +279,20 @@ class Pair:
         t = core.chop(self.t(),window=self.window)
         return t
     
+    def splitintens(self):
+        """
+        Calculate the splitting intensity as defined by Chevrot (2000).
+        """
+        copy = self.copy()
+        copy.rotateto(copy.pol)
+        copy.x = np.gradient(copy.x)
+        copy.chop()
+        rdiff, trans = copy.x, copy.y
+        s = -2 * np.trapz(trans * rdiff) / np.trapz(rdiff**2)
+        return s
+        
+    # Window
+    
     def wbeg(self):
         """
         Window start time.
@@ -371,6 +388,13 @@ class Pair:
         if self.window.width < self._nsamps():
             w1 = ax.axvline(self.wbeg(),linewidth=1,color='k')
             w2 = ax.axvline(self.wend(),linewidth=1,color='k')    
+        
+        # plot additional markers
+        if 'marker' in kwargs:
+            print('here')
+            if type(kwargs['marker']) is not list: kwargs['marker'] = [ kwargs['marker'] ]
+            [ ax.axvline(float(mark),linewidth=1,color='b') for mark in kwargs['marker'] ]
+            
         return
 
     def _ppm(self,ax,**kwargs):
@@ -539,21 +563,22 @@ class WindowPicker:
         # self.cidrelease = self.canvas.mpl_connect('button_release_event', self.release)
         self.cidenter = self.canvas.mpl_connect('axes_enter_event', self.enter)
         self.cidleave = self.canvas.mpl_connect('axes_leave_event', self.leave)
+        self.cidkey = self.canvas.mpl_connect('key_press_event', self.keypress) 
        
     def click(self,event):
         if event.inaxes is not self.ax: return
         x = event.xdata
-        d1 = math.fabs(self.x1 - x)
-        d2 = math.fabs(self.x2 - x)
         if event.button == 1:
-            if d1 < d2:
-                self.x1 = x
-                self.wbegline.set_data([x,x],self.ydat)
-            else:
-                self.x2 = x
-                self.wendline.set_data([x,x], self.ydat)
+            self.x1 = x
+            self.wbegline.set_data([x,x],self.ydat)
             self.canvas.draw() 
         if event.button == 3:
+            self.x2 = x
+            self.wendline.set_data([x,x], self.ydat)
+            self.canvas.draw()
+    
+    def keypress(self,event):
+        if event.key == " ":
             self.disconnect()
 
     def enter(self,event):
@@ -581,5 +606,6 @@ class WindowPicker:
         self.canvas.mpl_disconnect(self.cidenter)
         self.canvas.mpl_disconnect(self.cidleave)
         plt.close()
-        self.pair.set_window(self.x1, self.x2)
+        wbeg, wend = sorted((self.x1, self.x2)) 
+        self.pair.set_window(wbeg, wend)
       
\ No newline at end of file
